@@ -7,6 +7,8 @@ import posixpath
 import string
 import sys
 
+from collections import defaultdict
+
 from typing import Any, Dict, Optional, Union, Tuple
 
 from urllib.parse import urljoin, unquote, urlsplit
@@ -18,12 +20,13 @@ from pip_api.exceptions import PipError
 
 parser = argparse.ArgumentParser()
 parser.add_argument("req", nargs="*")
-parser.add_argument("-r", "--requirements")
+parser.add_argument("-r", "--requirement")
 parser.add_argument("-e", "--editable")
 # Consume index url params to avoid trying to treat them as packages.
 parser.add_argument("-i", "--index-url")
 parser.add_argument("--extra-index-url")
 parser.add_argument("-f", "--find-links")
+parser.add_argument("--hash", action="append", dest="hashes")
 
 operators = specifiers.Specifier._operators.keys()
 
@@ -37,6 +40,8 @@ WHEEL_FILE_RE = re.compile(
     re.VERBOSE,
 )
 WINDOWS = sys.platform.startswith("win") or (sys.platform == "cli" and os.name == "nt")
+# https://pip.pypa.io/en/stable/cli/pip_hash/
+VALID_HASHES = {"sha256", "sha384", "sha512"}
 
 
 class Link:
@@ -172,6 +177,15 @@ def _url_to_path(url):
     return path
 
 
+class Requirement(requirements.Requirement):
+    def __init__(self, *args, **kwargs):
+        self.hashes = kwargs.pop("hashes", None)
+        self.filename = kwargs.pop("filename")
+        self.lineno = kwargs.pop("lineno")
+
+        super().__init__(*args, **kwargs)
+
+
 class UnparsedRequirement(object):
     def __init__(self, name, msg, filename, lineno):
         self.name = name
@@ -445,8 +459,11 @@ def _parse_requirement_url(req_str):
 
 
 def parse_requirements(
-    filename: os.PathLike, options: Optional[Any] = None, include_invalid: bool = False
-) -> Dict[str, Union[requirements.Requirement, UnparsedRequirement]]:
+    filename: os.PathLike,
+    options: Optional[Any] = None,
+    include_invalid: bool = False,
+    strict_hashes: bool = False,
+) -> Dict[str, Union[Requirement, UnparsedRequirement]]:
     to_parse = {filename}
     parsed = set()
     name_to_req = {}
@@ -463,8 +480,20 @@ def parse_requirements(
         lines_enum = _skip_regex(lines_enum, options)
 
         for lineno, line in lines_enum:
-            req: Optional[Union[requirements.Requirement, UnparsedRequirement]] = None
+            req: Optional[Union[Requirement, UnparsedRequirement]] = None
             known, _ = parser.parse_known_args(line.strip().split())
+
+            hashes_by_kind = defaultdict(list)
+            if known.hashes:
+                for hsh in known.hashes:
+                    kind, hsh = hsh.split(":", 1)
+                    if kind not in VALID_HASHES:
+                        raise PipError(
+                            "Invalid --hash kind %s, expected one of %s"
+                            % (kind, VALID_HASHES)
+                        )
+                    hashes_by_kind[kind].append(hsh)
+
             if known.req:
                 req_str = str().join(known.req)
                 try:
@@ -477,7 +506,12 @@ def parse_requirements(
 
                 try:  # Try to parse this as a requirement specification
                     if req is None:
-                        req = requirements.Requirement(parsed_req_str)
+                        req = Requirement(
+                            parsed_req_str,
+                            hashes=dict(hashes_by_kind),
+                            filename=filename,
+                            lineno=lineno,
+                        )
                 except requirements.InvalidRequirement:
                     try:
                         _check_invalid_requirement(req_str)
@@ -487,13 +521,15 @@ def parse_requirements(
                         else:
                             raise
 
-            elif known.requirements:
-                full_path = os.path.join(dirname, known.requirements)
+            elif known.requirement:
+                full_path = os.path.join(dirname, known.requirement)
                 if full_path not in parsed:
                     to_parse.add(full_path)
             elif known.editable:
                 name, url = _parse_editable(known.editable)
-                req = requirements.Requirement("%s @ %s" % (name, url))
+                req = Requirement(
+                    "%s @ %s" % (name, url), filename=filename, lineno=lineno
+                )
             else:
                 pass  # This is an invalid requirement
 
@@ -512,4 +548,12 @@ def parse_requirements(
                         % (req, name_to_req[req.name], req.name)
                     )
 
+    if strict_hashes:
+        missing_hashes = [req for req in name_to_req.values() if not req.hashes]
+        if len(missing_hashes) > 0:
+            raise PipError(
+                "Missing hashes for requirement in %s, line %s"
+                % (missing_hashes[0].filename, missing_hashes[0].lineno)
+            )
+
     return name_to_req
